﻿using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using Infragistics.Windows.DockManager;
using Infragistics.Windows.DockManager.Events;

namespace PaneToolWindowExpansionSample
{
    public partial class MainWindow
    {
        private const string LoadedCounterFormat = "XamDockManager.ToolWindowLoaded: {0}";
        private const string ClosingCounterFormat = "PaneToolWindow.Closing: {0}";

        private int _loadedCounter;
        private int _closingCounter;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void XamDockManager_ToolWindowLoaded(object sender, PaneToolWindowEventArgs e)
        {
            IncreaseLoadedCounter();

            e.Window.Closing += WindowOnClosing;

            var control = FindVisualChildren<MyUserControl>(e.Window.Content as DependencyObject).FirstOrDefault();

            if (control != null)
            {
                e.Window.Width = double.NaN;
                e.Window.Height = double.NaN;
                e.Window.ResizeMode = ResizeMode.NoResize;
            }
        }

        private void WindowOnClosing(object sender, CancelEventArgs cancelEventArgs)
        {
            var toolWindow = (PaneToolWindow) sender;
            toolWindow.Closing -= WindowOnClosing;
            IncreaseClosingCounter();
        }

        private static IEnumerable<T> FindVisualChildren<T>(DependencyObject depObj) where T : DependencyObject
        {
            if (depObj != null)
            {
                for (int i = 0; i < VisualTreeHelper.GetChildrenCount(depObj); i++)
                {
                    DependencyObject child = VisualTreeHelper.GetChild(depObj, i);
                    if (child is T)
                    {
                        yield return (T) child;
                    }

                    foreach (T childOfChild in FindVisualChildren<T>(child))
                    {
                        yield return childOfChild;
                    }
                }
            }
        }

        private void OnWindowLoaded(object sender, RoutedEventArgs e)
        {
            var pane = new ContentPane
            {
                Content =
                    new MyUserControl()
                    {
                        VerticalAlignment = VerticalAlignment.Top,
                        HorizontalAlignment = HorizontalAlignment.Left
                    },
                Header = "New Tab"
            };

            TabGroupPane.Items.Add(pane);
        }

        private void IncreaseLoadedCounter()
        {
            _loadedCounter++;
            LoadedEventCounter.Content = string.Format(LoadedCounterFormat, _loadedCounter);
        }

        private void IncreaseClosingCounter()
        {
            _closingCounter++;
            ClosingEventCounter.Content = string.Format(ClosingCounterFormat, _closingCounter);
        }
    }
}
